@@ -8,10 +8,25 @@ use WWW::Crawler::Mojo::UserAgent;
 use Mojo::Message::Request;
 use Mojo::Util qw{md5_sum xml_escape dumper};
 use List::Util;
-our $VERSION = '0.08';
+our $VERSION = '0.09';
+
+our %tag_attributes = (
+    script  => ['src'],
+    link    => ['href'],
+    a       => ['href'],
+    img     => ['src'],
+    area    => ['href', 'ping'],
+    embed   => ['src'],
+    frame   => ['src'],
+    iframe  => ['src'],
+    input   => ['src'],
+    object  => ['data'],
+    form    => [sub {
+        _weave_form_data(@_);
+    }],
+);
 
 has active_conn => 0;
-has depth => 10;
 has fix => sub { {} };
 has active_conns_per_host => sub { {} };
 has max_conn => 1;
@@ -23,6 +38,7 @@ has 'ua' => sub { WWW::Crawler::Mojo::UserAgent->new };
 has 'ua_name' =>
     "www-crawler-mojo/$VERSION (+https://github.com/jamadam/www-crawler-mojo)";
 has 'shuffle';
+has tag_attributes => sub { \%tag_attributes };
 
 sub crawl {
     my ($self) = @_;
@@ -109,7 +125,6 @@ sub say_start {
 ----------------------------------------
 Crawling is starting with @{[ $self->queue->[0]->resolved_uri ]}
 Max Connection  : @{[ $self->max_conn ]}
-Depth           : @{[ $self->depth ]}
 User Agent      : @{[ $self->ua_name ]}
 EOF
 
@@ -153,7 +168,6 @@ sub scrape {
     my ($self, $res, $job) = @_;
     
     return if ($res->code != 200);
-    return if ($self->depth && $job->depth >= $self->depth);
     
     my $base = $job->resolved_uri;
     my $type = $res->headers->content_type;
@@ -164,18 +178,24 @@ sub scrape {
     }
     
     my $cb = sub {
-        my ($url, $dom) = @_;
+        my ($url, $dom, $method, $params) = @_;
         
-        $url =~ s{^\s*}{}g;
-        $url =~ s{\s*$}{}g;
+        $url = _clean_url_obj($url);
+        my $resolved = resolve_href($base, $url);
         
-        $url = Mojo::URL->new($url);
+        return unless ($resolved->scheme =~ qr{http|https|ftp|ws|wss});
         
-        return unless
-                (!$url->scheme || $url->scheme =~ qr{http|https|ftp|ws|wss});
+        my $child = $job->child(resolved_uri => $resolved, literal_uri => $url);
         
-        my $child = $job->child(
-            resolved_uri => resolve_href($base, $url), literal_uri => $url);
+        $child->method($method) if $method;
+        
+        if ($params) {
+            if ($method eq 'GET') {
+                $child->resolved_uri->query->append($params);
+            } else {
+                $child->tx_params($params);
+            }
+        }
         
         $self->emit('refer', sub {
             $self->enqueue($_[0] || $child);
@@ -185,13 +205,13 @@ sub scrape {
     if ($type && $type =~ qr{text/(html|xml)}) {
         my $encode = guess_encoding($res) || 'utf-8';
         my $body = Encode::decode($encode, $res->body);
-        collect_urls_html(Mojo::DOM->new($body), $cb);
+        $self->collect_urls_html(Mojo::DOM->new($body), $cb);
     }
     
     if ($type && $type =~ qr{text/css}) {
         my $encode  = guess_encoding($res) || 'utf-8';
         my $body    = Encode::decode($encode, $res->body);
-        collect_urls_css($body, $cb);
+        $self->collect_urls_css($body, $cb);
     }
 };
 
@@ -212,7 +232,9 @@ sub _enqueue {
             $job = WWW::Crawler::Mojo::Job->new(resolved_uri => $url);
         }
         
-        my $md5 = md5_sum($job->resolved_uri->to_string);
+        my $md5_seed = $job->resolved_uri->to_string. ($job->method || '');
+        $md5_seed .= $job->tx_params->to_string if ($job->tx_params);
+        my $md5 = md5_sum($md5_seed);
         if ($requeue || !exists $self->fix->{$md5}) {
             $self->fix->{$md5} = undef;
             push(@{$self->{queue}}, $job);
@@ -220,35 +242,16 @@ sub _enqueue {
     }
 }
 
-our %tag_attributes = (
-    script  => ['src'],
-    link    => ['href'],
-    a       => ['href'],
-    img     => ['src'],
-    area    => ['href', 'ping'],
-    embed   => ['src'],
-    frame   => ['src'],
-    iframe  => ['src'],
-    input   => ['src'],
-    object  => ['data'],
-    form    => ['action'],
-);
-
-sub _wrong_dom_detection {
-    my $dom = shift;
-    while ($dom = $dom->parent) {
-        return 1 if ($dom->type && $dom->type eq 'script');
-    }
-    return;
-}
-
 sub collect_urls_html {
-    my ($dom, $cb) = @_;
+    my ($self, $dom, $cb) = @_;
     
-    $dom->find(join(',', keys %tag_attributes))->each(sub {
+    $dom->find(join(',', keys %{$self->tag_attributes}))->each(sub {
         my $dom = shift;
         return if ($dom->xml && _wrong_dom_detection($dom));
-        for (@{$tag_attributes{$dom->type}}) {
+        for (@{$self->tag_attributes->{$dom->type}}) {
+            if (ref $_ && ref $_ eq 'CODE') {
+                $cb->($_->($dom));
+            }
             $cb->($dom->{$_}, $dom) if ($dom->{$_});
         }
     });
@@ -260,16 +263,16 @@ sub collect_urls_html {
     });
     $dom->find('style')->each(sub {
         my $dom = shift;
-        collect_urls_css($dom->content || '', sub { $cb->(shift, $dom) });
+        $self->collect_urls_css($dom->content || '', sub { $cb->(shift, $dom) });
     });
     $dom->find('*[style]')->each(sub {
         my $dom = shift;
-        collect_urls_css($dom->{style}, sub { $cb->(shift, $dom) });
+        $self->collect_urls_css($dom->{style}, sub { $cb->(shift, $dom) });
     });
 }
 
 sub collect_urls_css {
-    my ($str, $cb) = @_;
+    my ($self, $str, $cb) = @_;
     $str =~ s{/\*.+?\*/}{}gs;
     my @urls = ($str =~ m{url\(['"]?(.+?)['"]?\)}g);
     $cb->($_) for (@urls);
@@ -287,19 +290,6 @@ sub guess_encoding {
     return _guess_encoding_css($res->body) if ($type =~ qr{text/css});
 }
 
-sub _guess_encoding_css {
-    return (shift =~ qr{^\s*\@charset ['"](.+?)['"];}is)[0];
-}
-
-sub _guess_encoding_html {
-    my $head = (shift =~ qr{<head>(.+)</head>}is)[0] or return;
-    my $charset;
-    Mojo::DOM->new($head)->find('meta[http\-equiv=Content-Type]')->each(sub{
-        $charset = (shift->{content} =~ $charset_re)[0];
-    });
-    return $charset;
-}
-
 sub resolve_href {
     my ($base, $href) = @_;
     $href = ref $href ? $href : Mojo::URL->new($href);
@@ -312,6 +302,43 @@ sub resolve_href {
     return $abs;
 }
 
+sub _weave_form_data {
+    my $form = shift;
+    my %seed;
+    my $submit;
+    
+    $form->find("[name]")->each(sub {
+        my $e = shift;
+        my $name = $e->attr('name');
+        $seed{$name} ||= [];
+        
+        if ($e->type eq 'select') {
+            $e->find('option[selected=selected]')->each(sub {
+                push(@{$seed{$name}}, shift->attr('value'));
+            });
+        } elsif ($e->type eq 'textarea') {
+            push(@{$seed{$name}}, $e->text);
+        }
+        
+        my $type = $e->attr('type');
+        return unless $type;
+        
+        if (!$submit && grep{$_ eq $type} qw{submit image}) {
+            $submit = 1;
+            push(@{$seed{$name}}, $e->attr('value'));
+        } elsif (grep {$_ eq $type} qw{text hidden number}) {
+            push(@{$seed{$name}}, $e->attr('value'));
+        } elsif (grep {$_ eq $type} qw{checkbox}) {
+            push(@{$seed{$name}}, $e->attr('value')) if (exists $e->{checked});
+        } elsif (grep {$_ eq $type} qw{radio}) {
+            push(@{$seed{$name}}, $e->attr('value')) if (exists $e->{checked});
+        }
+    });
+    
+    return $form->{action}, $form,
+                    uc ($form->{method} || 'GET'), Mojo::Parameters->new(%seed);
+}
+
 sub _urls_redirect {
     my $tx = shift;
     my @urls;
@@ -320,6 +347,34 @@ sub _urls_redirect {
     return @urls;
 }
 
+sub _wrong_dom_detection {
+    my $dom = shift;
+    while ($dom = $dom->parent) {
+        return 1 if ($dom->type && $dom->type eq 'script');
+    }
+    return;
+}
+
+sub _clean_url_obj {
+    my $url = shift;
+    $url =~ s{^\s*}{}g;
+    $url =~ s{\s*$}{}g;
+    return Mojo::URL->new($url);
+}
+
+sub _guess_encoding_css {
+    return (shift =~ qr{^\s*\@charset ['"](.+?)['"];}is)[0];
+}
+
+sub _guess_encoding_html {
+    my $head = (shift =~ qr{<head>(.+)</head>}is)[0] or return;
+    my $charset;
+    Mojo::DOM->new($head)->find('meta[http\-equiv=Content-Type]')->each(sub{
+        $charset = (shift->{content} =~ $charset_re)[0];
+    });
+    return $charset;
+}
+
 sub _mod_busyness {
     my ($self, $uri, $inc) = @_;
     my $key = _host_key($uri);
@@ -391,6 +446,11 @@ moderate range of web pages so DO NOT use it for persistent crawler jobs.
 L<WWW::Crawler::Mojo> inherits all attributes from L<Mojo::EventEmitter> and
 implements the following new ones.
 
+=head2 tag_attributes
+
+Catalog of HTML attribute names which possibly contain URLs. Defaults to
+%__PACKAGE__::tag_attributes.
+
 =head2 ua
 
 A L<Mojo::UserAgent> instance.
@@ -419,15 +479,6 @@ A number of current connections per host.
     $bot->active_conns_per_host($bot->active_conns_per_host + 1);
     say $bot->active_conns_per_host;
 
-=head2 depth
-
-A number of max depth to crawl. Note that the depth is the number of HTTP
-requests to get to URI starting with the first job. This doesn't mean the
-deepness of URI path detected with slash.
-
-    $bot->depth(5);
-    say $bot->depth; # 5
-
 =head2 fix
 
 A hash whoes keys are md5 hashes of enqueued URLs.
@@ -619,7 +670,7 @@ enqueue method.
 
 Collects URLs out of HTML.
 
-    WWW::Crawler::Mojo::collect_urls_html($dom, sub {
+    $bot->collect_urls_html($dom, sub {
         my ($uri, $dom) = @_;
     });
 
@@ -627,7 +678,7 @@ Collects URLs out of HTML.
 
 Collects URLs out of CSS.
 
-    WWW::Crawler::Mojo::collect_urls_css($dom, sub {
+    $bot->collect_urls_css($dom, sub {
         my $uri = shift;
     });
 
